@@ -29,17 +29,18 @@ from __future__ import absolute_import, unicode_literals, division
 import copy
 import math
 import operator
-import sys
 from collections import deque, namedtuple
 from itertools import islice, repeat
 
-from . import modes as mo, graphics as g, charsets as cs
+from wcwidth import wcwidth
 
-
-if sys.version_info[0] == 2:
-    from future_builtins import map
-
-    range = xrange
+from . import (
+    charsets as cs,
+    control as ctrl,
+    graphics as g,
+    modes as mo
+)
+from .compat import map, range
 
 
 def take(n, iterable):
@@ -83,21 +84,26 @@ class Char(_Char):
     def __new__(cls, data, fg="default", bg="default", bold=False,
                 italics=False, underscore=False, reverse=False,
                 strikethrough=False):
-        return _Char.__new__(cls, data, fg, bg, bold, italics, underscore,
-                             reverse, strikethrough)
+        return super(Char, cls).__new__(cls, data, fg, bg, bold, italics,
+                                        underscore, strikethrough, reverse)
 
 
 class Cursor(object):
     """Screen cursor.
 
-    :param int x: horizontal cursor position.
-    :param int y: vertical cursor position.
+    :param int x: 0-based horizontal cursor position.
+    :param int y: 0-based vertical cursor position.
     :param pyte.screens.Char attrs: cursor attributes (see
-        :meth:`~pyte.screens.Screen.selectel_graphic_rendition`
+        :meth:`~pyte.screens.Screen.select_graphic_rendition`
         for details).
     """
+    __slots__ = ("x", "y", "attrs", "hidden")
+
     def __init__(self, x, y, attrs=Char(" ")):
-        self.x, self.y, self.attrs, self.hidden = x, y, attrs, False
+        self.x = x
+        self.y = y
+        self.attrs = attrs
+        self.hidden = False
 
 
 class Screen(object):
@@ -128,7 +134,7 @@ class Screen(object):
 
     .. note::
 
-       According to ``ECMA-48`` standard, **lines and columnns are
+       According to ``ECMA-48`` standard, **lines and columns are
        1-indexed**, so, for instance ``ESC [ 10;10 f`` really means
        -- move cursor to position (9, 9) in the display matrix.
 
@@ -142,28 +148,28 @@ class Screen(object):
     .. warning::
 
        If `DECAWM` mode is set than a cursor will be wrapped to the
-       **beginning* of the next line, which is the behaviour described
+       **beginning** of the next line, which is the behaviour described
        in ``man console_codes``.
 
     .. seealso::
 
        `Standard ECMA-48, Section 6.1.1 \
-       <http://www.ecma-international.org/publications
-       /standards/Ecma-048.htm>`_
-         For a description of the presentational component, implemented
-         by ``Screen``.
+       <http://www.ecma-international.org/publications/standards/Ecma-048.htm>`_
+       for a description of the presentational component, implemented
+       by ``Screen``.
     """
     #: A plain empty character with default foreground and background
     #: colors.
     default_char = Char(data=" ", fg="default", bg="default")
 
-    #: An inifinite sequence of default characters, used for populating
+    #: An infinite sequence of default characters, used for populating
     #: new lines and columns.
     default_line = repeat(default_char)
 
     def __init__(self, columns, lines):
         self.savepoints = []
-        self.lines, self.columns = lines, columns
+        self.columns = columns
+        self.lines = lines
         self.buffer = []
         self.reset()
 
@@ -185,11 +191,6 @@ class Screen(object):
         :param str command: command name, for example ``"LINEFEED"``.
         """
 
-    @property
-    def size(self):
-        """Returns screen size -- ``(lines, columns)``"""
-        return self.lines, self.columns
-
     @property
     def display(self):
         """Returns a :func:`list` of screen lines as unicode strings."""
@@ -354,7 +355,7 @@ class Screen(object):
         # Mark all displayed characters as reverse.
         if mo.DECSCNM in modes:
             self.buffer[:] = ([char._replace(reverse=True) for char in line]
-                       for line in self.buffer)
+                              for line in self.buffer)
             self.select_graphic_rendition(g._SGR["+reverse"])
 
         # Make the cursor visible.
@@ -367,7 +368,7 @@ class Screen(object):
         :param list modes: modes to reset -- hopefully, each mode is a
                            constant from :mod:`pyte.modes`.
         """
-        # Private mode codes are shifted, to be distingiushed from non
+        # Private mode codes are shifted, to be distinguished from non
         # private ones.
         if kwargs.get("private"):
             modes = [mode << 5 for mode in modes]
@@ -385,7 +386,7 @@ class Screen(object):
 
         if mo.DECSCNM in modes:
             self.buffer[:] = ([char._replace(reverse=False) for char in line]
-                       for line in self.buffer)
+                              for line in self.buffer)
             self.select_graphic_rendition(g._SGR["-reverse"])
 
         # Hide the cursor.
@@ -405,10 +406,23 @@ class Screen(object):
         the cursor if :data:`~pyte.modes.DECAWM` is set.
 
         :param str char: a character to display.
+
+        .. versionchanged:: 0.5.0
+
+           Character width is taken into account. Specifically, zero-width
+           and unprintable characters do not affect screen state. Full-width
+           characters are rendered into two consecutive character containers.
         """
         # Translating a given character.
-        char = char.translate([self.g0_charset,
-                               self.g1_charset][self.charset])
+        if self.charset:
+            char = char.translate(self.g1_charset)
+        else:
+            char = char.translate(self.g0_charset)
+
+        char_width = wcwidth(char)
+        if char_width <= 0:
+            # Unprintable character or doesn't advance the cursor.
+            return
 
         # If this was the last column in a line and auto wrap mode is
         # enabled, move the cursor to the beginning of the next line,
@@ -419,20 +433,23 @@ class Screen(object):
                 self.carriage_return()
                 self.linefeed()
             else:
-                self.cursor.x -= 1
+                self.cursor.x -= char_width
 
         # If Insert mode is set, new characters move old characters to
         # the right, otherwise terminal is in Replace mode and new
         # characters replace old characters at cursor position.
         if mo.IRM in self.mode:
-            self.insert_characters(1)
+            self.insert_characters(char_width)
 
-        self.buffer[self.cursor.y][self.cursor.x] = self.cursor.attrs \
-            ._replace(data=char)
+        line = self.buffer[self.cursor.y]
+        line[self.cursor.x] = self.cursor.attrs._replace(data=char)
+        if char_width > 1:
+            # Add a stub *after* a two-cell character. See issue #9 on GitHub.
+            line[self.cursor.x + 1] = self.cursor.attrs._replace(data=" ")
 
         # .. note:: We can't use :meth:`cursor_forward()`, because that
         #           way, we'll never know when to linefeed.
-        self.cursor.x += 1
+        self.cursor.x += char_width
 
     def carriage_return(self):
         """Move the cursor to the beginning of the current line."""
@@ -614,37 +631,38 @@ class Screen(object):
                             min(self.cursor.x + count, self.columns)):
             self.buffer[self.cursor.y][column] = self.cursor.attrs
 
-    def erase_in_line(self, type_of=0, private=False):
+    def erase_in_line(self, how=0, private=False):
         """Erases a line in a specific way.
 
-        :param int type_of: defines the way the line should be erased in:
+        :param int how: defines the way the line should be erased in:
 
             * ``0`` -- Erases from cursor to end of line, including cursor
               position.
             * ``1`` -- Erases from beginning of line to cursor,
               including cursor position.
             * ``2`` -- Erases complete line.
-        :param bool private: when ``True`` character attributes aren left
+        :param bool private: when ``True`` character attributes are left
                              unchanged **not implemented**.
         """
-        interval = (
+        if how == 0:
             # a) erase from the cursor to the end of line, including
-            # the cursor,
-            range(self.cursor.x, self.columns),
+            #    the cursor,
+            interval = range(self.cursor.x, self.columns)
+        elif how == 1:
             # b) erase from the beginning of the line to the cursor,
-            # including it,
-            range(0, self.cursor.x + 1),
+            #    including it,
+            interval = range(self.cursor.x + 1)
+        elif how == 2:
             # c) erase the entire line.
-            range(0, self.columns)
-        )[type_of]
+            interval = range(self.columns)
 
         for column in interval:
             self.buffer[self.cursor.y][column] = self.cursor.attrs
 
-    def erase_in_display(self, type_of=0, private=False):
+    def erase_in_display(self, how=0, private=False):
         """Erases display in a specific way.
 
-        :param int type_of: defines the way the line should be erased in:
+        :param int how: defines the way the line should be erased in:
 
             * ``0`` -- Erases from cursor to end of screen, including
               cursor position.
@@ -652,45 +670,47 @@ class Screen(object):
               including cursor position.
             * ``2`` -- Erases complete display. All lines are erased
               and changed to single-width. Cursor does not move.
-        :param bool private: when ``True`` character attributes aren left
+        :param bool private: when ``True`` character attributes are left
                              unchanged **not implemented**.
         """
-        interval = (
+        if how == 0:
             # a) erase from cursor to the end of the display, including
-            # the cursor,
-            range(self.cursor.y + 1, self.lines),
+            #    the cursor,
+            interval = range(self.cursor.y + 1, self.lines)
+        elif how == 1:
             # b) erase from the beginning of the display to the cursor,
-            # including it,
-            range(0, self.cursor.y),
+            #    including it,
+            interval = range(self.cursor.y)
+        elif how == 2:
             # c) erase the whole display.
-            range(0, self.lines)
-        )[type_of]
+            interval = range(self.lines)
 
         for line in interval:
             self.buffer[line][:] = \
                 (self.cursor.attrs for _ in range(self.columns))
 
         # In case of 0 or 1 we have to erase the line with the cursor.
-        if type_of in [0, 1]:
-            self.erase_in_line(type_of)
+        if how == 0 or how == 1:
+            self.erase_in_line(how)
 
     def set_tab_stop(self):
-        """Sest a horizontal tab stop at cursor position."""
+        """Sets a horizontal tab stop at cursor position."""
         self.tabstops.add(self.cursor.x)
 
-    def clear_tab_stop(self, type_of=None):
-        """Clears a horizontal tab stop in a specific way, depending
-        on the ``type_of`` value:
+    def clear_tab_stop(self, how=0):
+        """Clears a horizontal tab stop.
 
-        * ``0`` or nothing -- Clears a horizontal tab stop at cursor
-          position.
-        * ``3`` -- Clears all horizontal tab stops.
+        :param int how: defines a way the tab stop should be cleared:
+
+            * ``0`` or nothing -- Clears a horizontal tab stop at cursor
+              position.
+            * ``3`` -- Clears all horizontal tab stops.
         """
-        if not type_of:
+        if how == 0:
             # Clears a horizontal tab stop at cursor position, if it's
             # present, or silently fails if otherwise.
             self.tabstops.discard(self.cursor.x)
-        elif type_of == 3:
+        elif how == 3:
             self.tabstops = set()  # Clears all horizontal tab stops.
 
     def ensure_bounds(self, use_margins=None):
@@ -844,6 +864,44 @@ class Screen(object):
 
         self.cursor.attrs = self.cursor.attrs._replace(**replace)
 
+    def report_device_attributes(self):
+        """Reports terminal identity.
+
+        .. versionadded:: 0.5.0
+        """
+        # We only implement "primary" DA which is the only DA request
+        # VT102 understood, see ``VT102ID`` in ``linux/drivers/tty/vt.c``.
+        self.write_process_input(ctrl.CSI + "?6c")
+
+    def report_device_status(self, mode):
+        """Reports terminal status or cursor position.
+
+        :param int mode: if 5 -- terminal status, 6 -- cursor position,
+                         otherwise a noop.
+
+        .. versionadded:: 0.5.0
+        """
+        if mode == 5:    # Request for terminal status.
+            self.write_process_input(ctrl.CSI + "0n")
+        elif mode == 6:  # Request for cursor position.
+            x = self.cursor.x + 1
+            y = self.cursor.y + 1
+
+            # "Origin mode (DECOM) selects line numbering."
+            if mo.DECOM in self.mode:
+                y -= self.margins.top
+            self.write_process_input("{0}{1};{2}R".format(ctrl.CSI, y, x))
+
+    def write_process_input(self, data):
+        """Writes data to the process running inside the terminal.
+
+        By default is a noop.
+
+        :param str data: data to write to the process ``stdin``.
+
+        .. versionadded:: 0.5.0
+        """
+
 
 class DiffScreen(Screen):
     """A screen subclass, which maintains a set of dirty lines in its
@@ -883,8 +941,11 @@ class DiffScreen(Screen):
         super(DiffScreen, self).resize(*args, **kwargs)
 
     def draw(self, *args):
-        self.dirty.add(self.cursor.y)
+        # Call the superclass's method before marking the row as
+        # dirty, as when wrapping is enabled, draw() might change
+        # self.cursor.y.
         super(DiffScreen, self).draw(*args)
+        self.dirty.add(self.cursor.y)
 
     def index(self):
         if self.cursor.y == self.margins.bottom:
@@ -922,13 +983,15 @@ class DiffScreen(Screen):
         self.dirty.add(self.cursor.y)
         super(DiffScreen, self).erase_in_line(*args)
 
-    def erase_in_display(self, type_of=0):
-        self.dirty.update((
-            range(self.cursor.y + 1, self.lines),
-            range(0, self.cursor.y),
-            range(0, self.lines)
-        )[type_of])
-        super(DiffScreen, self).erase_in_display(type_of)
+    def erase_in_display(self, how=0):
+        if how == 0:
+            self.dirty.update(range(self.cursor.y + 1, self.lines))
+        elif how == 1:
+            self.dirty.update(range(self.cursor.y))
+        elif how == 2:
+            self.dirty.update(range(self.lines))
+
+        super(DiffScreen, self).erase_in_display(how)
 
     def alignment_display(self):
         self.dirty.update(range(self.lines))
